#ifndef _H_JCharEscape
#define _H_JCharEscape

/******************************************************************************
 JCharEscape.h

	Interface for the JCharEscape class.

	Copyright  1997 by Dustin Laurence.  All rights reserved.
	
	Base code generated by Codemill v0.1.0

 *****************************************************************************/

#include <JString.h>

class JCharEscape
{
public:

	JCharEscape();
	virtual ~JCharEscape();

	// The reason the class exists
	void Substitute(JString* string) const;
	void operator()(JString* string) const;

	// Useful shortcuts for building tables
	void AddNonprintingEscapes();
	void AddWhitespaceEscapes();
	void AddCEscapes();
	void AddRegexExtensions();

	// Build your own single-character escape table by hand
	JBoolean SetEscape(const unsigned char character, const JString& value);
	JBoolean SetEscape(const unsigned char character, const JCharacter* value, const JSize length);
	JBoolean SetEscape(const unsigned char character, const JCharacter* value);

	const JString* GetEscape(const unsigned char character) const;
	JBoolean EscapeExists(const unsigned char character) const;

	JBoolean ClearEscape(const unsigned char character);

	void ClearAllOrdinary();

	// Support for special kinds of multi-character escapes
	void     SetControlEscapes(const JBoolean yesNo = kTrue);
	JBoolean GetControlEscapes() const;

	// Choose default behavior for unrecognized escapes
	void     SetIdentityDefault(const JBoolean enable = kTrue);
	JBoolean GetIdentityDefault() const;

	void Reset();

private:

	JString**   itsSubTable;
	JBoolean  itsControlEscapesFlag;
	JBoolean  itsIdentityDefaultFlag;

private:

	// not allowed

	JCharEscape(const JCharEscape& source);
	const JCharEscape& operator=(const JCharEscape& source);
};

/******************************************************************************
 operator()

	This operator is simply an alternate interface for Substitute and
	allows the class as a functor (a function object or smart function).
	It's a bit of syntactic sugar, but nice for a class built around a
	single function.

 *****************************************************************************/

inline void
JCharEscape::operator()
	(
	JString* string
	)
	const
{
	Substitute(string);
}

/******************************************************************************
 GetEscape

	Returns a pointer to the internal string value (for copying) if it
	exists, or NULL otherwise.

 *****************************************************************************/

inline const JString*
JCharEscape::GetEscape
	(
	const unsigned char character
	)
	const
{
	return itsSubTable[character];
}

/******************************************************************************
 EscapeExists

	Returns kTrue if an escape exists for the given character, kFalse otherwise.

 *****************************************************************************/

inline JBoolean
JCharEscape::EscapeExists
	(
	const unsigned char character
	)
	const
{
	return JI2B(itsSubTable[character] != NULL);
}

/******************************************************************************
 ControlEscapes

	This flag enables special Perl-style two-character escapes for control
	characters; "\cX" is replaced with the control-X character.  The
	default is no control escapes.  When enabled '\c' obeys the special
	rules below; when disabled it obeys the ordinary rules.  Any ordinary
	value bound to '\c' is untouched.

	'X' must be one of a capital letter, '[', '\', ']', '^', or '_'; the
	escape sequence is replaced with the ASCII value 'X' - 64. Otherwise
	the escape is treated as an ordinary unrecognized escape (regardless of
	whether '\c' has a separate setting) and thus will be given the default
	treatment (if identity, only the backslash is removed, not the 'c').

	Note that control-@ or '\c@' is disallowed because it represents ASCII
	NUL, the same as C's NULL string terminator '\0'.  It didn't seem like
	a good idea to make it easier to insert NULL into the middle of a
	string.  If you really want to insert NULL give '\0' a value with
	SetEscape() and a length-counted pointer or a JString.

 *****************************************************************************/

inline void
JCharEscape::SetControlEscapes
	(
	const JBoolean enable // = kTrue
	)
{
	itsControlEscapesFlag = enable;
}

inline JBoolean
JCharEscape::GetControlEscapes() const
{
	return itsControlEscapesFlag;
}

/******************************************************************************
 IdentityDefault

	This flag controls the behavior when an escape is not recognized.
	kFalse (the default) indicates that unknown escapes should be ignored,
	leaving the backslash in place.  kTrue means that the backslash should
	be removed, so that unrecognized escapes just represent the escaped
	character itself.

 *****************************************************************************/

inline void
JCharEscape::SetIdentityDefault
	(
	const JBoolean yesNo // = kTrue
	)
{
	itsIdentityDefaultFlag = yesNo;
}

inline JBoolean
JCharEscape::GetIdentityDefault() const
{
	return itsIdentityDefaultFlag;
}

#endif
