/******************************************************************************
 JFileArrayIndex.h

	Interface for JFileArrayIndex class (used by JFileArray class)

	Copyright  1998 John Lindal. All rights reserved.

 ******************************************************************************/

#ifndef _H_JFileArrayIndex
#define _H_JFileArrayIndex

#include <JContainer.h>
#include <JFileArray.h>		// need JFAIndex, JFAID
#include <JArray.h>

// types of elements stored in JFileArray

enum FAElementType
	{
	kData         = 0,
	kEmbeddedFile = 1
	};

class JFileArrayIndex : public JContainer
{
public:

	JFileArrayIndex();

	virtual	~JFileArrayIndex();

	void	InsertElementAtIndex(const JFAIndex& index, const JUnsignedOffset offset,
								 const JFAID& id);
	void	RemoveElement(const JFAIndex& index, const JSize elementSize);

	void	MoveElementToIndex(const JFAIndex& currentIndex, const JFAIndex& newIndex);
	void	SwapElements(const JFAIndex& index1, const JFAIndex& index2);

	JUnsignedOffset	GetElementOffset(const JFAIndex& index) const;
	void			ElementSizeChanged(const JFAIndex& index,
									   const JInteger changeInElementSize);

	JFAID	GetElementID(const JFAIndex& index) const;
	void	SetElementID(const JFAIndex& index, const JFAID& id);

	JBoolean	GetElementIndexFromID(const JFAID& id, JFAIndex* index) const;
	JFAID		GetUniqueID() const;

	void		SetToEmbeddedFile(const JFAIndex& index);
	JBoolean	IsEmbeddedFile(const JFAIndex& index) const;

	void		EmbeddedFileOpened(const JFAIndex& index, JFileArray* theEmbeddedFile);
	void		EmbeddedFileClosed(const JFAIndex& index);

	JBoolean	EmbeddedFileIsClosed(const JFAIndex& index) const;
	JBoolean	AllEmbeddedFilesAreClosed() const;

	void		ReplaceEmbeddedFileStreams(fstream* newStream);

	void		ReadIndex(const JSize elementCount, istream& input);
	void		WriteIndex(ostream& output);
	JSize		GetIndexLength() const;

private:

	#include <ElementInfo.h>

private:

	JArray<ElementInfo>*	itsArray;

private:

	// not allowed

	JFileArrayIndex(const JFileArrayIndex& source);
	JFileArrayIndex& operator=(const JFileArrayIndex& source);
};

/******************************************************************************
 MoveElementToIndex

 ******************************************************************************/

inline void
JFileArrayIndex::MoveElementToIndex
	(
	const JFAIndex& currentIndex,
	const JFAIndex& newIndex
	)
{
	itsArray->MoveElementToIndex(currentIndex.GetIndex(), newIndex.GetIndex());
}

/******************************************************************************
 SwapElements

 ******************************************************************************/

inline void
JFileArrayIndex::SwapElements
	(
	const JFAIndex& index1,
	const JFAIndex& index2
	)
{
	itsArray->SwapElements(index1.GetIndex(), index2.GetIndex());
}

#endif
