/******************************************************************************
 JLineProtocol.h

	Interface for event handler classes.

	Copyright  1998 by John Lindal. All rights reserved.

 ******************************************************************************/

#ifndef _H_JLineProtocol
#define _H_JLineProtocol

#include <JPtrArray-JString.h>
#include <ace/Svc_Handler.h>
#include <ace/Synch_T.h>

class JLineProtocolT
{
public:

	// Reserved characters

	static const JCharacter kMsgSeparatorChar;
	static const JCharacter kDisconnectChar;

	// Other constants

	enum
		{
		kDefaultBufferSize = 1000
		};

public:

	// Broadcaster messages

	static const JCharacter* kLineReady;
	static const JCharacter* kReceivedDisconnect;	// other end sent disconnect

	class LineReady : public JBroadcaster::Message
		{
		public:

			LineReady(const JString& line)
				:
				JBroadcaster::Message(kLineReady),
				itsLine(line)
				{ };

			// This does not remove the line from the input queue.

			const JString&
			PeekLine() const
			{
				return itsLine;
			};

		private:

			const JString& itsLine;
		};

	class ReceivedDisconnect : public JBroadcaster::Message
		{
		public:

			ReceivedDisconnect()
				:
				JBroadcaster::Message(kReceivedDisconnect)
				{ };
		};
};

template <ACE_PEER_STREAM_1>
class JLineProtocol : public ACE_Svc_Handler<ACE_PEER_STREAM_2, ACE_SYNCH>,
					  virtual public JBroadcaster
{
public:

	JLineProtocol();
	JLineProtocol(const ACE_HANDLE fd);

	virtual ~JLineProtocol();

	JBoolean	ReceivedDisconnect() const;
	void		SendDisconnect();

	JBoolean	HasLines() const;
	JSize		GetLineCount() const;
	JBoolean	GetNextLine(JString* line);
	JBoolean	PeekNextLine(JString* line);
	JBoolean	PeekPartialLine(JString* line);

	void		SendLine(const JString& line);
	void		SendLine(const JCharacter* line);
	void		SendLine(const JCharacter* line, const JSize length);

	void		SendString(const JString& str);
	void		SendString(const JCharacter* str);
	void		SendString(const JCharacter* str, const JSize length);

	// ACE_Svc_Handler functions

	virtual int	open(void*);
	virtual int	handle_input(ACE_HANDLE);
	virtual int handle_close(ACE_HANDLE, ACE_Reactor_Mask);

private:

	JPtrArray<JString>*	itsMessageList;		// parsed messages, last one is incomplete

	JCharacter*	itsBuffer;					// buffer to receive raw bytes
	JSize		itsBufferSize;

	JBoolean	itsSentDisconnectFlag;		// kTrue => we are finished
	JBoolean	itsReceivedDisconnectFlag;	// kTrue => other end is finished

private:

	void		JLineProtocolX();
	JString*	NewMessage();

	// not allowed

	JLineProtocol(const JLineProtocol& source);
	JLineProtocol& operator=(const JLineProtocol& source);
};

#endif
