/******************************************************************************
 JPrinter.h

	Interface for the JPrinter class

	Copyright  1996 by John Lindal. All rights reserved.

 ******************************************************************************/

#ifndef _H_JPrinter
#define _H_JPrinter

#include <JPainter.h>

class JPrinter : public JPainter
{
public:

	JPrinter(const JFontManager* fontManager, const JColormap* colormap,
			 const JRect& defaultClipRect);

	virtual ~JPrinter();

	virtual JBoolean	WillPrintBlackWhite() const = 0;

	// page info

	JIndex		GetPageIndex() const;

	JCoordinate	GetPageWidth() const;
	JCoordinate	GetPageHeight() const;
	JRect		GetPageRect() const;
	void		SetPageSize(const JCoordinate w, const JCoordinate h);

	void		GetPageInfo(JIndex* pageIndex, JCoordinate* w, JCoordinate* h) const;

	// printing control

	virtual void		OpenDocument() = 0;
	virtual JBoolean	NewPage() = 0;
	virtual void		CloseDocument() = 0;
	virtual void		CancelDocument() = 0;

	// page header and footer

	void	LockHeader(const JCoordinate headerHeight);
	void	LockFooter(const JCoordinate footerHeight);

protected:

	void	SetPageIndex(const JIndex pageIndex);

private:

	JIndex		itsPageIndex;
	JCoordinate	itsPageWidth;
	JCoordinate	itsPageHeight;

private:

	// not allowed

	JPrinter(const JPrinter& source);
	const JPrinter& operator=(const JPrinter& source);
};


/******************************************************************************
 Page info

 ******************************************************************************/

inline JIndex
JPrinter::GetPageIndex()
	const
{
	return itsPageIndex;
}

inline void
JPrinter::SetPageIndex		// protected
	(
	const JIndex pageIndex
	)
{
	itsPageIndex = pageIndex;
}

inline JCoordinate
JPrinter::GetPageWidth()
	const
{
	return itsPageWidth;
}

inline JCoordinate
JPrinter::GetPageHeight()
	const
{
	return itsPageHeight;
}

inline JRect
JPrinter::GetPageRect()
	const
{
	return JRect(0, 0, itsPageHeight, itsPageWidth);
}

inline void
JPrinter::SetPageSize
	(
	const JCoordinate w,
	const JCoordinate h
	)
{
	itsPageWidth  = w;
	itsPageHeight = h;
}

inline void
JPrinter::GetPageInfo
	(
	JIndex*			pageIndex,
	JCoordinate*	w,
	JCoordinate*	h
	)
	const
{
	*pageIndex = itsPageIndex;
	*w = itsPageWidth;
	*h = itsPageHeight;
}

#endif
