/******************************************************************************
 jDirUtil.h

	Utility functions for dealing with directories.

	Copyright  1996 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#ifndef _H_jDirUtil
#define _H_jDirUtil

#include <JStdError.h>
#include <time.h>

#if __MWERKS__
	#include <stat.h>
#elif defined __VCPP__
	#include <sys/stat.h>
	typedef unsigned short mode_t;
#else
	#include <sys/stat.h>
	#include <sys/types.h>
#endif

class JString;
class JProgressDisplay;

JBoolean	JNameUsed(const JCharacter* name);
JError		JGetModificationTime(const JCharacter* name, time_t* modTime);
JError		JGetPermissions(const JCharacter* name, mode_t* perms);
JError		JSetPermissions(const JCharacter* name, const mode_t perms);

JBoolean	JDirectoryExists(const JCharacter* dirName);
JBoolean	JDirectoryReadable(const JCharacter* dirName);
JBoolean	JDirectoryWritable(const JCharacter* dirName);
JBoolean	JCanEnterDirectory(const JCharacter* dirName);

JError		JCreateDirectory(const JCharacter* dirName);
JError		JCreateDirectory(const JCharacter* dirName, const int mode);
JError		JRenameDirectory(const JCharacter* oldName, const JCharacter* newName);
JError		JChangeDirectory(const JCharacter* dirName);
JError		JRemoveDirectory(const JCharacter* dirName);
JBoolean	JKillDirectory(const JCharacter* dirName);

JString		JGetCurrentDirectory();
JBoolean	JGetHomeDirectory(JString* homeDir);
JBoolean	JGetTrueName(const JCharacter* name, JString* trueName);
JBoolean	JSearchSubdirs(const JCharacter* startPath, const JCharacter* name,
						   const JBoolean isFile, const JBoolean caseSensitive,
						   JString* path, JString* newName = NULL,
						   JProgressDisplay* pg = NULL);

JBoolean	JProgramOnPath(const JCharacter* programName);

JString		JOpenTempDirectory(const JCharacter* dirName = NULL);
void		JCloseTempDirectory(const JCharacter* dirName);

JString		JCombinePathAndName(const JCharacter* path, const JCharacter* name);
void		JSplitPathAndName(const JCharacter* file, JString* path, JString* name);
void		JAppendDirSeparator(JString* dirName);

#endif
