/******************************************************************************
 jTypes.h

	Defines portable types

	Copyright  1994-97 by John Lindal. All rights reserved.

 ******************************************************************************/

#ifndef _H_jTypes
#define _H_jTypes

#include <limits.h>
#include <float.h>

class istream;
class ostream;

#ifndef NULL
#define NULL	0L
#endif

	// general types

typedef char								JCharacter;

const JCharacter kJCharacterMin				= CHAR_MIN;
const JCharacter kJCharacterMax				= (JCharacter) CHAR_MAX;	// cast required for VCPP

typedef long								JInteger;

const JInteger kJIntegerMin					= LONG_MIN;
const JInteger kJIntegerMax					= LONG_MAX;

typedef unsigned long						JUInt;

const JInteger kJUIntMin					= 0;
const JInteger kJUIntMax					= ULONG_MAX;

typedef double								JFloat;

const JFloat kJFloatMin						= DBL_MIN;
const JFloat kJFloatMax						= DBL_MAX;

const long kJFloatExponentMin				= DBL_MIN_10_EXP;
const long kJFloatExponentMax				= DBL_MAX_10_EXP;

	// specialized types

typedef unsigned long						JSize;

const JSize kJSizeMin						= 0;
const JSize kJSizeMax						= ULONG_MAX;

typedef unsigned long						JIndex;

const JIndex kJIndexMin						= 0;
const JIndex kJIndexMax						= ULONG_MAX;

typedef long								JSignedIndex;

const JSignedIndex kJSignedIndexMin			= LONG_MIN;
const JSignedIndex kJSignedIndexMax			= LONG_MAX;

typedef unsigned long						JUnsignedOffset;

const JUnsignedOffset kJUnsignedOffsetMin	= 0;
const JUnsignedOffset kJUnsignedOffsetMax	= ULONG_MAX;

typedef long								JSignedOffset;

const JSignedOffset kJSignedOffsetMin		= LONG_MIN;
const JSignedOffset kJSignedOffsetMax		= LONG_MAX;

typedef long								JCoordinate;

const JCoordinate kJCoordinateMin			= LONG_MIN;
const JCoordinate kJCoordinateMax			= LONG_MAX;

typedef unsigned long						JFileVersion;

const JFileVersion kJFileVersionMin			= 0;
const JFileVersion kJFileVersionMax			= ULONG_MAX;

	// used mainly by JHistogram.tmpl

typedef JIndex								JBinIndex;

const JBinIndex kJBinIndexMin				= kJIndexMin;
const JBinIndex kJBinIndexMax				= kJIndexMax;

	// boolean type

enum JBoolean
{
	kFalse = 0,
	kTrue  = 1
};

const JSize kJBooleanDataStreamLength = sizeof(JCharacter);

istream& operator>>(istream& input, JBoolean& jbool);
ostream& operator<<(ostream& output, const JBoolean jbool);

inline JBoolean
JConvertToBoolean
	(
	const int comparison
	)
{
	return (comparison ? kTrue : kFalse);
}

inline JBoolean
JI2B
	(
	const int comparison
	)
{
	return (comparison ? kTrue : kFalse);
}

inline JBoolean
JNegate
	(
	const int comparison
	)
{
	return (comparison ? kFalse : kTrue);
}

// This is provided because 0.1 should be considered "true" since
// it is non-zero.  JConvertToBoolean() would truncate it to zero.
// Of course, you shouldn't be using floating-point values to store
// flags, but somebody else might force you to.

inline JBoolean
JFloatToBoolean
	(
	const double x
	)
{
	return (x == 0.0 ? kFalse : kTrue);
}

inline JBoolean
JF2B
	(
	const double x
	)
{
	return (x == 0.0 ? kFalse : kTrue);
}

/******************************************************************************
 JUInt32

	An unsigned integer guaranteed to be 32 bits on all supported machines, a
	bit like the FORTRAN (gasp!) *N style idiom.  JUInt32 is similar to an
	unsigned version of a FORTRAN INTEGER*4.

	Considerations:

	All unix machines seem to define an int as 32 bits, regardless of wordsize,
	while a long is the machine's native wordsize, 32 bits on 32-bit architectures
	and 64 on 64-bit processors like the Alpha.  This is consistent with the
	traditional (i.e. before 64-bit processor) C programmer's assumption that

	pointers and longs can convert back and forth without loss of information.

	MetroWerks uses 16-bit ints (perhaps the original Mac had 16-bit words?),
	so on the Mac we use a long.

 ******************************************************************************/

#ifdef __MWERKS__
	typedef unsigned long JUInt32;
#else
	typedef unsigned int  JUInt32;
#endif

/******************************************************************************
 JInt32

	A 32-bit signed integer.  The considerations for JUInt32 also apply here, of
	course.

	For strict ANSI compliance it can be a good idea to use a signed integer
	when the quantity may be interpreted as either a signed or unsigned value.
	The reason for this is that if a signed integral quantity is converted to
	an unsigned quantity ANSI guarantees that it behaves exactly as you think
	it should, i.e. as though the signed quantity was stored in two's complement
	representation and the bit pattern was simply re-interpreted as an unsigned
	integer.

	By contrast, ANSI does *not* guarantee that the reverse is true.  An unsigned
	quantity converted to a signed quantity overflows when its value is too large
	to fit into the signed quantity, *and the result is undefined*.  The fact
	that every C programmer on the planet behaves as though ANSI mandates the
	usual unix/two's complement behavior of re-interpreting the bit pattern as a
	two's complement value apparently did not sway the standards committee, and
	code which depends on such behavior is not strictly portable.  On the other
	hand, I wonder how many C compliers would dare to break this behavior.

 ******************************************************************************/

#ifdef __MWERKS__
	typedef signed long JInt32;
#else
	typedef signed int  JInt32;
#endif

/******************************************************************************

	Defines a standard hash value type, e.g. for use in JHashTable.

 ******************************************************************************/

// JHashValue must be unsigned to guarantee that >> shifts in zero bits

typedef JUInt32 JHashValue;

// JDualHashValue must be signed so the step can go either way

typedef JInt32 JDualHashValue;

#endif
