/******************************************************************************
 JXCSFDialogBase.h

	Interface for the JXCSFDialogBase class

	Copyright  1996 by John Lindal. All rights reserved.

 ******************************************************************************/

#ifndef _H_JXCSFDialogBase
#define _H_JXCSFDialogBase

#include <JXDialogDirector.h>

class JString;
class JUNIXDirInfo;
class JXUNIXDirTable;
class JXWidget;
class JXStaticText;
class JXStringInput;
class JXPathInput;
class JXTextButton;
class JXTextCheckbox;
class JXStringHistoryMenu;
class JXPathHistoryMenu;
class JXScrollbarSet;
class JXGetNewDirDialog;
class JXIdleTask;

class JXCSFDialogBase : public JXDialogDirector
{
public:

	JXCSFDialogBase(JXDirector* supervisor, JUNIXDirInfo* dirInfo,
					const JCharacter* fileFilter);

	virtual ~JXCSFDialogBase();

	virtual void	Activate();

	const JString&	GetPath() const;
	const JString&	GetFilter() const;
	JBoolean		HiddenVisible() const;

	// called by JXChooseSaveFile

	virtual void	GetUpdateTasks(JPtrArray<JXIdleTask>* taskList);
													// must call inherited

	void	ReadBaseSetup(istream& input);
	void	WriteBaseSetup(ostream& output) const;

protected:

	void	SetObjects(JXScrollbarSet* scrollbarSet,
					   JXStaticText* pathLabel, JXPathInput* pathInput,
					   JXPathHistoryMenu* pathHistory,
					   JXStaticText* filterLabel, JXStringInput* filterInput,
					   JXStringHistoryMenu* filterHistory,
					   JXTextButton* enterButton,
					   JXTextButton* upButton, JXTextButton* homeButton,
					   JXTextButton* newDirButton, JXTextCheckbox* showHiddenCB,
					   const JCharacter* message);

	virtual void	AdjustSizings();	// must call inherited
	virtual void	UpdateDisplay();	// must call inherited

	JUNIXDirInfo*	GetDirInfo() const;
	JXUNIXDirTable*	GetFileBrowser() const;
	JXPathInput*	GetPathInput() const;
	JXStringInput*	GetFilterInput() const;
	JXTextButton*	GetNewDirButton() const;

	JBoolean	GoToItsPath();
	void		AdjustFilter();

	virtual void	Receive(JBroadcaster* sender, const Message& message);

private:

	JUNIXDirInfo*	itsDirInfo;			// we don't own this
	JString*		itsPrevPath;
	JString*		itsPrevFilterString;

	JXUNIXDirTable*			itsFileBrowser;
	JXPathHistoryMenu*		itsPathHistory;
	JXPathInput*			itsPathInput;
	JXStringHistoryMenu*	itsFilterHistory;
	JXStringInput*			itsFilterInput;
	JXTextButton*			itsEnterButton;
	JXTextButton*			itsUpButton;
	JXTextButton*			itsHomeButton;
	JXTextButton*			itsNewDirButton;	// can be NULL
	JXTextCheckbox*			itsShowHiddenCB;

	JXGetNewDirDialog*	itsNewDirDialog;

private:

	void	DisplayMessage(const JCharacter* message, JXScrollbarSet* scrollbarSet,
						   JXStaticText* pathLabel, JXPathHistoryMenu* pathHistory,
						   JXStaticText* filterLabel, JXStringHistoryMenu* filterHistory);

	void	GetNewDirectory();
	void	CreateNewDirectory();

	void	SelectPrevDirectory();

	// not allowed

	JXCSFDialogBase(const JXCSFDialogBase& source);
	const JXCSFDialogBase& operator=(const JXCSFDialogBase& source);
};


/******************************************************************************
 Access to objects (protected)

 ******************************************************************************/

inline JUNIXDirInfo*
JXCSFDialogBase::GetDirInfo()
	const
{
	return itsDirInfo;
}

inline JXUNIXDirTable*
JXCSFDialogBase::GetFileBrowser()
	const
{
	return itsFileBrowser;
}

inline JXPathInput*
JXCSFDialogBase::GetPathInput()
	const
{
	return itsPathInput;
}

inline JXStringInput*
JXCSFDialogBase::GetFilterInput()
	const
{
	return itsFilterInput;
}

inline JXTextButton*
JXCSFDialogBase::GetNewDirButton()
	const
{
	return itsNewDirButton;
}

#endif
