/******************************************************************************
 JXFileHistoryMenu.h

	Interface for the JXFileHistoryMenu class

	Copyright  1998 by John Lindal. All rights reserved.

 ******************************************************************************/

#ifndef _H_JXFileHistoryMenu
#define _H_JXFileHistoryMenu

#include <JXTextMenu.h>

class JXFileHistoryMenu : public JXTextMenu
{
public:

	JXFileHistoryMenu(const JSize historyLength,
					  const JCharacter* title, JXContainer* enclosure,
					  const HSizingOption hSizing, const VSizingOption vSizing,
					  const JCoordinate x, const JCoordinate y,
					  const JCoordinate w, const JCoordinate h);

	JXFileHistoryMenu(const JSize historyLength,
					  JXMenu* owner, const JIndex itemIndex, JXContainer* enclosure);

	virtual ~JXFileHistoryMenu();

	JSize	GetHistoryLength() const;
	void	SetHistoryLength(const JSize historyLength);
	void	ClearHistory();

	void	AddFile(const JCharacter* fullName);
	void	AddFile(const JCharacter* path, const JCharacter* name);

	JString	GetFile(const JIndex index) const;

	void	SetDefaultIcon(JXImage* icon, const JBoolean menuOwnsIcon);

	void	ReadSetup(istream& input);
	void	WriteSetup(ostream& output) const;

protected:

	virtual void	ShowIcon();

	virtual void	Receive(JBroadcaster* sender, const Message& message);

private:

	JSize		itsHistoryLength;
	JXImage*	itsDefFileIcon;
	JBoolean	itsOwnsFileIconFlag;

private:

	void	JXFileHistoryMenuX(const JSize historyLength);
	void	AdjustLength();
	void	RemoveNonexistentFiles();

	// not allowed

	JXFileHistoryMenu(const JXFileHistoryMenu& source);
	const JXFileHistoryMenu& operator=(const JXFileHistoryMenu& source);
};


/******************************************************************************
 GetHistoryLength

 ******************************************************************************/

inline JSize
JXFileHistoryMenu::GetHistoryLength()
	const
{
	return itsHistoryLength;
}

/******************************************************************************
 ClearHistory

 ******************************************************************************/

inline void
JXFileHistoryMenu::ClearHistory()
{
	RemoveAllItems();
	Deactivate();
}

#endif
