/******************************************************************************
 JXIconMenu.h

	Interface for the JXIconMenu class

	Copyright  1996 by John Lindal. All rights reserved.

 ******************************************************************************/

#ifndef _H_JXIconMenu
#define _H_JXIconMenu

#include <JXMenu.h>

class JXImage;
class JXIconMenuData;

class JXIconMenu : public JXMenu
{
public:

	JXIconMenu(const JCharacter* title, const JSize columnCount,
			   JXContainer* enclosure,
			   const HSizingOption hSizing, const VSizingOption vSizing,
			   const JCoordinate x, const JCoordinate y,
			   const JCoordinate w, const JCoordinate h);

	JXIconMenu(JXImage* image, const JBoolean menuOwnsImage,
			   const JSize columnCount, JXContainer* enclosure,
			   const HSizingOption hSizing, const VSizingOption vSizing,
			   const JCoordinate x, const JCoordinate y,
			   const JCoordinate w, const JCoordinate h);

	JXIconMenu(const JSize columnCount, JXMenu* owner, const JIndex itemIndex,
			   JXContainer* enclosure);

	virtual ~JXIconMenu();

	void			InsertItem(const JIndex index, JXImage* image,
							   const JBoolean menuOwnsImage,
							   const JBoolean isCheckbox = kFalse,
							   const JBoolean isRadio = kFalse);
	void			PrependItem(JXImage* image, const JBoolean menuOwnsImage,
								const JBoolean isCheckbox = kFalse,
								const JBoolean isRadio = kFalse);
	void			AppendItem(JXImage* image, const JBoolean menuOwnsImage,
							   const JBoolean isCheckbox = kFalse,
							   const JBoolean isRadio = kFalse);

	const JXImage*	GetItemImage(const JIndex index) const;
	void			SetItemImage(const JIndex index, JXImage* image,
								 const JBoolean menuOwnsImage);

protected:

	virtual JXMenuDirector*	CreateWindow(JXWindowDirector* supervisor);
	virtual void			AdjustPopupChoiceTitle(const JIndex index);

	JXIconMenuData*	GetIconMenuData();

private:

	JXIconMenuData*	itsIconMenuData;

private:

	void	JXIconMenuX(const JSize columnCount);

	// not allowed

	JXIconMenu(const JXIconMenu& source);
	const JXIconMenu& operator=(const JXIconMenu& source);
};


/******************************************************************************
 GetIconMenuData (protected)

 ******************************************************************************/

inline JXIconMenuData*
JXIconMenu::GetIconMenuData()
{
	return itsIconMenuData;
}

#endif
