/******************************************************************************
 JXIconMenuData.h

	Interface for the JXIconMenuData class

	Copyright  1996 by John Lindal. All rights reserved.

 ******************************************************************************/

#ifndef _H_JXIconMenuData
#define _H_JXIconMenuData

#include <JXMenuData.h>
#include <JArray.h>

class JXImage;
class JXIconMenuTable;

class JXIconMenuData : public JXMenuData
{
public:

	JXIconMenuData(const JSize columnCount);

	virtual ~JXIconMenuData();

	void			InsertItem(const JIndex index, JXImage* image,
							   const JBoolean menuOwnsImage,
							   const JBoolean isCheckbox = kFalse,
							   const JBoolean isRadio = kFalse);
	void			PrependItem(JXImage* image, const JBoolean menuOwnsImage,
								const JBoolean isCheckbox = kFalse,
								const JBoolean isRadio = kFalse);
	void			AppendItem(JXImage* image, const JBoolean menuOwnsImage,
							   const JBoolean isCheckbox = kFalse,
							   const JBoolean isRadio = kFalse);
	virtual void	DeleteItem(const JIndex index);		// must call inherited
	virtual void	DeleteAll();						// must call inherited

	const JXImage*	GetImage(const JIndex index) const;
	void			SetImage(const JIndex index, JXImage* image,
							 const JBoolean menuOwnsImage);

	JSize	GetColumnCount() const;
	void	SetColumnCount(const JSize columnCount);

	// called by JXIconMenuTable

	void	ConfigureTable(JXIconMenuTable* table,
						   JBoolean* hasCheckboxes, JBoolean* hasSubmenus);

private:

	struct IconData
	{
		JXImage*	image;
		JBoolean	ownsImage;	// kTrue if we should delete image

		IconData()
			:
			image( NULL ), ownsImage( kTrue )
		{ };

		IconData(JXImage* i, const JBoolean own)
			:
			image( i ), ownsImage( own )
		{ };
	};

private:

	JArray<IconData>*	itsIconData;
	JSize				itsColumnCount;

	JBoolean			itsNeedGeomRecalcFlag;
	JCoordinate			itsRowHeight;
	JCoordinate			itsColWidth;

private:

	void	CleanOutIconItem(IconData* itemData);

	// not allowed

	JXIconMenuData(const JXIconMenuData& source);
	const JXIconMenuData& operator=(const JXIconMenuData& source);

public:

	// JBroadcaster messages

	static const JCharacter* kImageChanged;

	class ImageChanged : public JBroadcaster::Message
		{
		public:

			ImageChanged(const JIndex index)
				:
				JBroadcaster::Message(kImageChanged),
				itsIndex(index)
			{ };

			JIndex
			GetIndex() const
			{
				return itsIndex;
			};

		private:

			JIndex itsIndex;
		};
};


/******************************************************************************
 New item

 ******************************************************************************/

inline void
JXIconMenuData::PrependItem
	(
	JXImage*		image,
	const JBoolean	menuOwnsImage,
	const JBoolean	isCheckbox,
	const JBoolean	isRadio
	)
{
	InsertItem(1, image, menuOwnsImage, isCheckbox, isRadio);
}

inline void
JXIconMenuData::AppendItem
	(
	JXImage*		image,
	const JBoolean	menuOwnsImage,
	const JBoolean	isCheckbox,
	const JBoolean	isRadio
	)
{
	InsertItem(GetElementCount()+1, image, menuOwnsImage, isCheckbox, isRadio);
}

/******************************************************************************
 GetImage

 ******************************************************************************/

inline const JXImage*
JXIconMenuData::GetImage
	(
	const JIndex index
	)
	const
{
	const IconData itemData = itsIconData->GetElement(index);
	return itemData.image;
}

/******************************************************************************
 Item count per row

 ******************************************************************************/

inline JSize
JXIconMenuData::GetColumnCount()
	const
{
	return itsColumnCount;
}

inline void
JXIconMenuData::SetColumnCount
	(
	const JSize columnCount
	)
{
	itsColumnCount = columnCount;
}

#endif
