/******************************************************************************
 JXPathInput.h

	Interface for the JXPathInput class

	Copyright  1996 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#ifndef _H_JXPathInput
#define _H_JXPathInput

#include <JXStringInput.h>

class JXPathInput : public JXStringInput
{
public:

	JXPathInput(JXContainer* enclosure,
				const HSizingOption hSizing, const VSizingOption vSizing,
				const JCoordinate x, const JCoordinate y,
				const JCoordinate w, const JCoordinate h);

	virtual ~JXPathInput();

	virtual JBoolean	InputValid();

	JBoolean	IsAllowingInvalidPath() const;
	void		AllowInvalidPath(const JBoolean allow = kTrue);

	static JColorIndex	GetTextColor(const JCharacter* path, const JColormap* colormap);

protected:

	virtual void	HandleUnfocusEvent();
	virtual void	ApertureResized(const JCoordinate dw, const JCoordinate dh);
	virtual void	Receive(JBroadcaster* sender, const Message& message);

	virtual void	AdjustStylesBeforeRecalc(const JString& buffer, JRunArray<Font>* styles,
											 JIndexRange* recalcRange, JIndexRange* redrawRange,
											 const JBoolean deletion);

private:

	JBoolean	itsAllowInvalidPathFlag;

private:

	// not allowed

	JXPathInput(const JXPathInput& source);
	const JXPathInput& operator=(const JXPathInput& source);
};


/******************************************************************************
 Allow invalid path

 ******************************************************************************/

inline JBoolean
JXPathInput::IsAllowingInvalidPath()
	const
{
	return itsAllowInvalidPathFlag;
}

inline void
JXPathInput::AllowInvalidPath
	(
	const JBoolean allow
	)
{
	itsAllowInvalidPathFlag = allow;
}

#endif
