/******************************************************************************
 JXTextClipboard.h

	Interface for the JXTextClipboard class

	Copyright  1997 by John Lindal. All rights reserved.

 ******************************************************************************/

#ifndef _H_JXTextClipboard
#define _H_JXTextClipboard

#include <JXTEBase.h>

class JXTextClipboard : public JXTEBase
{
public:

	JXTextClipboard(const JBoolean copyStyledText, JXContainer* enclosure);

	virtual ~JXTextClipboard();

	JBoolean	SetClipboard(const JCharacter* text);
	JBoolean	UpdateClipboard();

	JBoolean	SetClipboard(const JCharacter* text, const JBoolean copyStyles);
	JBoolean	UpdateClipboard(const JBoolean copyStyles);

	JBoolean	WillCopyStyles() const;
	void		ShouldCopyStyles(const JBoolean copyStyles);

private:

	JBoolean	GrabClipboard();

	// not allowed

	JXTextClipboard(const JXTextClipboard& source);
	const JXTextClipboard& operator=(const JXTextClipboard& source);
};


/******************************************************************************
 SetClipboard

	Place the given text on the clipboard.  Returns kTrue if successful.

 ******************************************************************************/

inline JBoolean
JXTextClipboard::SetClipboard
	(
	const JCharacter* text
	)
{
	Paste(text);
	return GrabClipboard();
}

inline JBoolean
JXTextClipboard::SetClipboard
	(
	const JCharacter*	text,
	const JBoolean		copyStyles
	)
{
	const JBoolean savedCopyStyles = WillPasteStyledText();
	ShouldPasteStyledText(copyStyles);
	Paste(text);
	ShouldPasteStyledText(savedCopyStyles);
	return GrabClipboard();
}

/******************************************************************************
 UpdateClipboard

	If there is text on the clipboard, this copies it and then grabs
	ownership of the clipboard.  Returns kTrue if there was text on
	the clipboard.

 ******************************************************************************/

inline JBoolean
JXTextClipboard::UpdateClipboard()
{
	Paste();				// get the clipboard contents
	return GrabClipboard();
}

inline JBoolean
JXTextClipboard::UpdateClipboard
	(
	const JBoolean copyStyles
	)
{
	const JBoolean savedCopyStyles = WillPasteStyledText();
	ShouldPasteStyledText(copyStyles);
	Paste();				// get the clipboard contents
	ShouldPasteStyledText(savedCopyStyles);
	return GrabClipboard();
}

/******************************************************************************
 Copy styled text

 ******************************************************************************/

inline JBoolean
JXTextClipboard::WillCopyStyles()
	const
{
	return WillPasteStyledText();
}

inline void
JXTextClipboard::ShouldCopyStyles
	(
	const JBoolean copyStyles
	)
{
	ShouldPasteStyledText(copyStyles);
}

#endif
