/******************************************************************************
 ItemCost.h

	Interface for ItemCost class.

	Written by John Lindal.

 ******************************************************************************/

#ifndef _H_ItemCost
#define _H_ItemCost

#include <JFileHashTable.h>

class JString;

class ItemCost : public JFileHashTable<JString>
{
public:

	ItemCost();

	virtual ~ItemCost();

	void	AddItem(const JString& theKey, const JFloat cost);
	void	RemoveItem(const JString& theKey);

	JBoolean	GetItemCost(const JString& theKey, JFloat* cost);
	JBoolean	SetItemCost(const JString& theKey, const JFloat cost);

	void		LockForIteration();
	JBoolean	GetNextItem(JString* theKey, JFloat* cost);
	void		UnlockAfterIteration();

private:

	JBoolean	itsLockForIterFlag;	// kTrue if we are iterating over all items
	JIndex		itsHashTableIndex;	// index for iterating through hash table

private:

	virtual JIndex		Hash(const JString& theKey) const;
	virtual JBoolean	CompareFromFile(const JString& theKey) const;
	virtual void		InitializeElement(const JString& theKey);

	JString	ReadKey() const;
	void	WriteKey(const JString& theKey);

	JFloat	ReadCost() const;
	void	WriteCost(const JFloat cost);

	// not allowed

	ItemCost(const ItemCost&);
	const ItemCost& operator=(const ItemCost& source);
};


/******************************************************************************
 UnlockAfterIteration

 ******************************************************************************/

inline void
ItemCost::UnlockAfterIteration()
{
	itsLockForIterFlag = kFalse;
}

#endif
