/******************************************************************************
 TestChooseSaveFile.cc

	Class to test overriding of JXChooseSaveFile dialog windows.

	BASE CLASS = JXChooseSaveFile

	Written by John Lindal.

 ******************************************************************************/

#include "TestChooseSaveFile.h"
#include "TestSaveFileDialog.h"
#include <jAssert.h>

/******************************************************************************
 Constructor

 ******************************************************************************/

TestChooseSaveFile::TestChooseSaveFile()
	:
	JXChooseSaveFile()
{
	itsSaveFileDialog = NULL;
	itsSaveFormat     = kGIFFormat;
}

/******************************************************************************
 Destructor

 ******************************************************************************/

TestChooseSaveFile::~TestChooseSaveFile()
{
}

/******************************************************************************
 CreateSaveFileDialog (virtual protected)

 ******************************************************************************/

JXSaveFileDialog*
TestChooseSaveFile::CreateSaveFileDialog
	(
	JXDirector*			supervisor,
	JUNIXDirInfo*		dirInfo,
	const JCharacter*	fileFilter,
	const JCharacter*	origName,
	const JCharacter*	prompt,
	const JCharacter*	message
	)
{
	itsSaveFileDialog =
		new TestSaveFileDialog(supervisor, dirInfo, fileFilter, itsSaveFormat);
	assert( itsSaveFileDialog != NULL );
	itsSaveFileDialog->BuildWindow(origName, prompt, message);
	return itsSaveFileDialog;
}

/******************************************************************************
 GetSaveFormat

 ******************************************************************************/

TestChooseSaveFile::SaveFormat
TestChooseSaveFile::GetSaveFormat()
	const
{
	return itsSaveFormat;
}

/******************************************************************************
 Receive (protected)

	Listen for response from itsSaveFileDialog.

 ******************************************************************************/

void
TestChooseSaveFile::Receive
	(
	JBroadcaster*	sender,
	const Message&	message
	)
{
	if (sender == itsSaveFileDialog && message.Is(JXDialogDirector::kDeactivated))
		{
		const JXDialogDirector::Deactivated* info =
			dynamic_cast(const JXDialogDirector::Deactivated*, &message);
		assert( info != NULL );
		if (info->Successful())
			{
			itsSaveFormat = itsSaveFileDialog->GetSaveFormat();
			}
		itsSaveFileDialog = NULL;
		}

	JXChooseSaveFile::Receive(sender, message);
}
